@@ -7,6 +7,33 @@ mediaShow.NOT_LOADED = 0;
 mediaShow.LOADING = 1;
 mediaShow.LOADED = 2;
 
+//Hash cache to check for changes
+mediaShow.lastHash = "";
+mediaShow.hashCheckRunning = false;
+
+//Start Checking for hash changes every second
+mediaShow.startHashCheck = function ()
+{
+  if(!mediaShow.hashCheckRunning)
+  {
+    mediaShow.hashCheckRunning = true;
+    
+    setInterval(function() {
+      if(mediaShow.lastHash != window.location.hash) {
+          mediaShow.hashChanged();
+          mediaShow.lastHash = window.location.hash;
+      }
+    }, 500);
+  }
+}
+
+
+//Run if hash has changed
+mediaShow.hashChanged = function ()
+{
+  mediaShow.readURLAndUpdate();
+}
+
 //Fixed height of the slideshows
 mediaShow.containerHeight = 480;
 
@@ -27,7 +54,8 @@ mediaShow.findSlideshows = function ()
                                    "currentSlide": 0,
                                    "slides": new Array(),
                                    "initialized": false,
-                                   "size": 0
+                                   "size": 0,
+                                   "hash": ""
                                   });
     });  
 };
@@ -46,7 +74,8 @@ mediaShow.getContentListing = function (slideshow)
         $.each(data, function(index, item) {
             //-------------------- Declaration of Slide ------------------
             slideshow.slides.push({
-                "url":item,
+                "url":item.url,
+                "UID" : item.UID,
                 "loaded": mediaShow.NOT_LOADED
                 });
             slideshow.size++;
@@ -198,6 +227,8 @@ mediaShow.markAsInitialized = function (slideshow)
     slideshow.obj.find(".mediaShowSlide_"+slideshow.currentSlide).show();
     
     slideshow.initialized = true;
+    mediaShow.hashChanged();
+    mediaShow.startHashCheck();
     mediaShow.startLoading(slideshow);
 };
 
@@ -233,8 +264,8 @@ mediaShow.next = function (slideshowIndex)
 {
   var slideshow = mediaShow.slideshows[slideshowIndex];
   if(slideshow.currentSlide + 1 <= slideshow.size - 1)
-  {
-    mediaShow.goToSlide(slideshow.currentSlide + 1, slideshow);
+  { 
+    mediaShow.goToSlide(slideshow.currentSlide + 1, slideshow); 
   }
   else
   {
@@ -260,27 +291,100 @@ mediaShow.prev = function (slideshowIndex)
   return false;
 }
 
+//This returns the index of the slide with the provided UID.
+mediaShow.idToIndex = function (slideshow, uid)
+{
+  var found = -1;
+  $.each(slideshow.slides, function(index, slide){
+    if(slide.UID == uid)
+    {
+      found = index;
+    }
+  });
+  return found;
+};
+
+//This reads the URL hash and updates the slideshows acordingly
+mediaShow.readURLAndUpdate = function ()
+{
+  var hash = document.location.hash;
+  if(hash == "")
+    return;
+  
+  var hash_split = hash.substring(1,hash.length).split(",");
+  $.each(hash_split, function(index, hsh){
+    $.each(mediaShow.slideshows, function(index, slideshow){
+      var slideIndex = mediaShow.idToIndex(slideshow, hsh);
+      if (slideIndex > -1)
+      {
+        slideshow.hash = hsh;
+        mediaShow.goToSlide(slideIndex, slideshow);
+        return false;
+      }
+      return true;
+    });
+  });
+};
+
+//This function will update the browser's url to the current state of the multiple slideshows.
+mediaShow.updateURL = function (slideshow, slideNumber)
+{
+  var uid = slideshow.slides[slideNumber].UID;
+  var old_hash = document.location.hash;
+  var new_hash = uid;
+  var hash_split = old_hash.substring(1,old_hash.length).split(",");
+  var hash = new Array();
+  
+  var replaced = false;
+  $.each(hash_split, function(index, hsh){
+    if(hsh == slideshow.hash)
+    {
+      hash.push(new_hash);
+      replaced = true;
+    }
+    else
+    {
+      hash.push(hsh);
+    }
+  });
+  
+  if(!replaced)
+  {
+    hash.push(new_hash);
+  }
+  
+  document.location.hash = hash.join(",");
+  slideshow.hash = new_hash;
+};
+
 //This shows slide number x on the given slideshow
 mediaShow.goToSlide = function (x, slideshow)
 {
-  slideshow.obj.find(".mediaShowSlide_"+x).show();
-  var sizeOfContainer = mediaShow.containerHeight;
-  var img = slideshow.obj.find(".mediaShowSlide_"+x).find('img')[0];
-  height = $(img).attr('offsetHeight');
-  if(height > 0 && height < sizeOfContainer)
+  mediaShow.updateURL(slideshow, x);
+  if(slideshow.slides[x].loaded == mediaShow.NOT_LOADED)
   {
-    var margin = (sizeOfContainer - height)/2;
-    $(img).css('margin-top', margin);
+    slideshow.currentSlide = x;
+  }
+  {
+    slideshow.obj.find(".mediaShowSlide_"+x).show();
+    var sizeOfContainer = mediaShow.containerHeight;
+    var img = slideshow.obj.find(".mediaShowSlide_"+x).find('img')[0];
+    height = $(img).attr('offsetHeight');
+    if(height > 0 && height < sizeOfContainer)
+    {
+      var margin = (sizeOfContainer - height)/2;
+      $(img).css('margin-top', margin);
+    }
+    
+    slideshow.currentSlide = x;
+    
+    $.each(slideshow.slides, function (index, slide) {
+                                if(index != x)
+                                {
+                                  slideshow.obj.find(".mediaShowSlide_"+index).hide();
+                                }
+                              });
   }
-  
-  slideshow.currentSlide = x;
-  
-  $.each(slideshow.slides, function (index, slide) {
-                              if(index != x)
-                              {
-                                slideshow.obj.find(".mediaShowSlide_"+index).hide();
-                              }
-                            });
 }
 
 //This is the first function to run, finds slideshows on the page and initializes them
