@@ -94,7 +94,7 @@ class MediaShowListingView(BrowserView):
         jsonStr = "["
         for res in results:
             if(self.getMediaURL(res.getObject()) != ""):
-                jsonStr += '"'+res.getURL()+'"'
+                jsonStr += '{"url" : "'+res.getURL()+'", "UID" : "'+res["UID"]+'"}'
                 jsonStr += ","
             
         if jsonStr != "[":
